package org.wso2.registry.web.actions;

import org.wso2.registry.Collection;
import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.servlet.utils.UserUtil;
import org.wso2.registry.web.MediaTypesReader;
import org.wso2.registry.web.actions.utils.ResourceData;
import org.wso2.registry.web.actions.utils.ResourcePath;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: chanaka jayasena
 * Date: May 11, 2008
 * Time: 1:47:55 PM
 * To change this template use File | Settings | File Templates.
 */
/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */





public class ShowResourceTreeAction extends AbstractRegistryAction{

//    private Log log = LogFactory.getLog(CollectionViewAction.class);

    private String path;
    private List resourceDataList = new ArrayList();
    private String parentPath;
    private String status;
    private String errorMessage;
    private List navigatablePaths = new ArrayList();
    private List collectionTypes = new ArrayList();
    private String resourceTree = "";
    private int resourceTreeIndex = 1;
    private String textBoxId;

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        // get the resource for the path
        // get all its children and fill ResourceOverview objects
        // add ResourceOverview object to the resourceDataList list

        Registry registry = getRegistry();

        Map<String,String[]> rootChildren= fetchChildren(RegistryConstants.ROOT_PATH);
        Iterator iRootChildren = rootChildren.keySet().iterator();
        while(iRootChildren.hasNext()){
            String next = (String)iRootChildren.next();
            fillResourceTree(next,rootChildren.get(next));
        }
        //Fill the resourceTree String with html data


        // if path is not set, we assume that root "/" is requested
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
        }

        if (path.equals(RegistryConstants.ROOT_PATH)) {

            ResourcePath resourcePath = new ResourcePath();
            resourcePath.setNavigateName("root");
            resourcePath.setNavigatePath("#");

            navigatablePaths.add(resourcePath);

        } else {

            // first add the root path
            ResourcePath rootPath = new ResourcePath();
            rootPath.setNavigateName("root");
            rootPath.setNavigatePath(RegistryConstants.ROOT_PATH);
            navigatablePaths.add(rootPath);

            String preparedPath = path;
            if (preparedPath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(RegistryConstants.PATH_SEPARATOR.length());
            }

            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.
                        substring(0, preparedPath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            String[] parts = preparedPath.split(RegistryConstants.PATH_SEPARATOR);
            for (int i = 0; i < parts.length; i++) {
                ResourcePath resourcePath = new ResourcePath();
                resourcePath.setNavigateName(parts[i]);

                String tempPath = "";
                for (int j = 0; j < i + 1; j++) {
                    tempPath = tempPath + RegistryConstants.PATH_SEPARATOR + parts[j];
                }
                resourcePath.setNavigatePath(tempPath);
                navigatablePaths.add(resourcePath);
            }
        }



        // construct the parent path
        if (path.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = "root";
        } else {
            String formattedPath = path;
            if (path.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = path.substring(
                        0, path.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        Resource resource ;
        try {
            resource = registry.get(path);
        } catch (AuthorizationFailedException e) {
            status = "error";
            errorMessage = "User " + getUserName() +
                    " does not have permission to browse this collection.";
            return ERROR;
        }

        if (resource == null) {
            status = "error";
            errorMessage = "Requested collection at path " + path + " does not exists.";
            return ERROR;
        }

        String[] childPaths = (String[]) resource.getContent();
        for (String childPath : childPaths) {

            try {
                Resource child = registry.get(childPath);

                ResourceData resourceData = new ResourceData();
                resourceData.setResourcePath(childPath);

                if (childPath != null) {
                    String[] parts = childPath.split(RegistryConstants.PATH_SEPARATOR);
                    resourceData.setName(parts[parts.length - 1]);
                }

                resourceData.setResourceType(child instanceof Collection ?
                        ResourceData.COLLECTION : ResourceData.RESOURCE);
                resourceData.setAuthorUserName(child.getAuthorUserName());
                resourceData.setDescription(child.getDescription());
                resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                resourceData.setCreatedOn(child.getCreatedTime());
                resourceData.setDeleteAllowed(
                        UserUtil.isDeleteAllowed(getUserName(), childPath, request));
                calculateAverageStars(resourceData);

                resourceDataList.add(resourceData);

            } catch (AuthorizationFailedException ignore) {
                // if we get an auth failed exception while accessing a child, we simply skip it.
                // we are not showing unauthorized resources.
            }


        }

        collectionTypes = MediaTypesReader.getCollectionMediaTypes(request);

        status = "success";
        return SUCCESS;
    }
    public void fillResourceTree(String resourceName, String[] resourceData){
        resourceTreeIndex++;
        resourceTree += "<div class=\"father-object\">";
        if(resourceData[1]!=ResourceData.RESOURCE) {
                        resourceTree +="<a onclick=\"showHideCommon('y_plus_"+resourceTreeIndex+"');showHideCommon('y_minus_"+resourceTreeIndex+"');showHideCommon('z_"+resourceTreeIndex+"');\">" +
                            "<img src=\"/wso2registry/admin/images/icon-tree-plus.jpg\" id=\"y_plus_"+resourceTreeIndex+"\" style=\"display:none;margin-right:5px;\" />" +
                            "<img src=\"/wso2registry/admin/images/icon-tree-minus.jpg\" id=\"y_minus_"+resourceTreeIndex+"\" style=\"margin-right:5px;\" />" +
                            "<a onclick=\"pickPath('"+ resourceData[0] + "')\">" + resourceName +
                        "</a>";        
        } else {
             resourceTree += "<a class=\"plane-resource\" onclick=\"pickPath('"+ resourceData[0] + "')\">" + resourceName + "</a>";
        }
        resourceTree += "</div>"+ "<div class=\"child-bojects\" id=\"z_"+resourceTreeIndex+"\">";
		if(resourceData[1]==ResourceData.RESOURCE) {
            resourceTree +="</div>";
            return;
        }
        else {
			if(resourceData[2]=="false"){
                resourceTree +="</div>";
                return;
            } else{
				Map<String,String[]> grandChildren= fetchChildren(resourceData[0]);
                Iterator iRootChildren = grandChildren.keySet().iterator();
                while(iRootChildren.hasNext()){
                    String next = (String)iRootChildren.next();
                    //Recursive call
                    fillResourceTree(next,grandChildren.get(next));
                }
            }
        }
        resourceTree +="</div>";
    }
    public Map<String, String[]> fetchChildren(String queryPath){

        Map<String, String[]> childcollection= new HashMap();
        try {

            Registry registry = getRegistry();
            String[] childPaths;
            Resource resource;
            resource = registry.get(queryPath);
            childPaths = (String[])resource.getContent();
            for(String childPath: childPaths){
                String childName = "";
                String hasChildren = "false";

                if (childPath != null) {
                    String[] parts = childPath.split(RegistryConstants.PATH_SEPARATOR);
                    childName = parts[parts.length - 1];
                }
                Resource child = registry.get(childPath);
                String childResourceType = child instanceof Collection ?  ResourceData.COLLECTION : ResourceData.RESOURCE;
                resource = registry.get(childPath);
                if(resource.getContent()!=null) hasChildren = "true";
                String[] childData = {childPath,childResourceType,hasChildren };

                childcollection.put(childName,childData);
            }
        } catch (RegistryException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        }

        return childcollection;
    }
    public String getPath() {
        return path;
    }

    public void setPath(String path) {

        // we are handling only the current version in the human view
        //if (path.indexOf("?") > 0) {
        //    path = path.split("\\?")[0];
        //}

        this.path = path;
    }
    public String getResourceTree() {
        return resourceTree;
    }

    public void setResourceTree(String resourceTree) {
        this.resourceTree = resourceTree;
    }
    public List getResourceDataList() {
        return resourceDataList;
    }

    public void setResourceDataList(List resourceDataList) {
        this.resourceDataList = resourceDataList;
    }

    public String getParentPath() {
        return parentPath;
    }

    public void setParentPath(String parentPath) {
        this.parentPath = parentPath;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public void setErrorMessage(String errorMessage) {
        this.errorMessage = errorMessage;
    }

    public List getNavigatablePaths() {
        return navigatablePaths;
    }

    public void setNavigatablePaths(List navigatablePaths) {
        this.navigatablePaths = navigatablePaths;
    }

    public List getCollectionTypes() {
        return collectionTypes;
    }

    public void setCollectionTypes(List collectionTypes) {
        this.collectionTypes = collectionTypes;
    }
    public String getTextBoxId() {
        return textBoxId;
    }

    public void setTextBoxId(String textBoxId) {
        this.textBoxId = textBoxId;
    }
    private void calculateAverageStars(ResourceData resourceData) {

        float tempRating = resourceData.getAverageRating() * 1000;
        tempRating = Math.round(tempRating);
        tempRating = tempRating / 1000;
        resourceData.setAverageRating(tempRating);

        float averageRating = resourceData.getAverageRating();
        String[] averageStars = new String[5];

        for (int i = 0; i < 5; i++) {

            if (averageRating >= i + 1) {
                averageStars[i] = "04";

            } else if (averageRating <= i) {
                averageStars[i] = "00";

            } else {

                float fraction = averageRating - i;

                if (fraction <= 0.125) {
                    averageStars[i] = "00";

                } else if (fraction > 0.125 && fraction <= 0.375) {
                    averageStars[i] = "01";

                } else if (fraction > 0.375 && fraction <= 0.625) {
                    averageStars[i] = "02";

                } else if (fraction > 0.625 && fraction <= 0.875) {
                    averageStars[i] = "03";

                } else {
                    averageStars[i] = "04";

                }
            }
        }

        resourceData.setAverageStars(averageStars);
    }
}

